@@ -38,6 +38,7 @@ Box_Data_List::Box_Data_List()
     if(dialog)
     {
       m_pDialogLayout = dialog;
+      add_view(m_pDialogLayout); //Give it access to the document.
       m_pDialogLayout->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
     }
   }
@@ -45,6 +46,7 @@ Box_Data_List::Box_Data_List()
   m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [Add] or enter data into the last row to add a new record.\n Leave automatic ID fields empty - they will be filled for you.\nOnly the first 100 records are shown.");
 
   pack_start(m_AddDel);
+  add_view(&m_AddDel); //Give it access to the document.
   m_AddDel.set_auto_add(false); //We want to add the row ourselves when the user clicks the Add button, because the default behaviour there is not suitable.
   m_AddDel.set_rules_hint(); //Use alternating row colors when the theme does that.
 
@@ -67,6 +69,7 @@ Box_Data_List::Box_Data_List()
 
 Box_Data_List::~Box_Data_List()
 {
+  remove_view(&m_AddDel);
 }
 
 void Box_Data_List::fill_from_database()
@@ -77,12 +80,12 @@ void Box_Data_List::fill_from_database()
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
 
-    Box_DB_Table::fill_from_database();
+    Box_Data::fill_from_database();
 
     m_AddDel.remove_all();
 
     //Field Names:
-    fill_column_titles();
+    //create_layout();
 
     //if(sharedconnection)
     //{
@@ -95,78 +98,11 @@ void Box_Data_List::fill_from_database()
     m_AddDel.set_allow_add(table_privs.m_create);
     m_AddDel.set_allow_delete(table_privs.m_delete);
 
+    m_AddDel.set_where_clause(m_strWhereClause);
+
     if(table_privs.m_view)
     {
-      Glib::ustring strWhereClausePiece;
-      if(!m_strWhereClause.empty())
-        strWhereClausePiece = " WHERE " + m_strWhereClause;
-
-      m_Fields = get_fields_to_show();
-
-      //Add extra possibly-non-visible columns that we need:
-      type_vecLayoutFields fieldsToGet = m_Fields;
-
-      LayoutItem_Field layout_item;
-      Field field_key = m_AddDel.get_key_field();
-      layout_item.set_name(field_key.get_name());
-      layout_item.m_field = field_key;
-      fieldsToGet.push_back(layout_item);
-      const int index_primary_key = fieldsToGet.size() - 1;
-
-      if(!fieldsToGet.empty())
-      {
-        const Glib::ustring query = util_build_sql_select_with_where_clause(m_strTableName, fieldsToGet, m_strWhereClause);
-
-        Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
-        if(!result)
-          handle_error();
-        else
-        {
-          //Field contents:
-          m_AddDel.remove_all();
-
-          //each row:
-          guint rows_count = result->get_n_rows();
-          if(rows_count)
-            m_has_one_or_more_records = true; //Save it for later.
-
-          if(rows_count > 100)
-            rows_count = 100; //Don't get more than 100. TODO: Get other rows dynamically.
-
-          for(guint result_row = 0; result_row < (rows_count); result_row++)
-          {
-            Gnome::Gda::Value key = result->get_value_at(index_primary_key, result_row);
-            //It is usually an integer.
-            if(GlomConversions::value_is_empty(key))
-              g_warning("Box_Data_List::fill_from_database(): primary key value is empty");
-            else
-            {
-              Gnome::Gda::Value value_primary_key = result->get_value_at(index_primary_key, result_row);
-
-              Gtk::TreeModel::iterator tree_iter = m_AddDel.add_item(value_primary_key);
-
-              type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
-
-              //each field:
-              //We use cols_count -1 because we have an extra field for the primary_key.
-              guint cols_count = result->get_n_columns() - 1;
-              for(guint uiCol = 0; uiCol < cols_count; uiCol++)
-              {
-                const Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
-                const LayoutItem_Field& layout_item = *iterFields;
-
-                //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
-                //g_warning("  value_as_string=%s", value.to_string().c_str());
-
-                m_AddDel.set_value(tree_iter, layout_item, value);
-                //g_warning("addedel size=%d", m_AddDel.get_count());
-
-                ++iterFields;
-              }
-            }
-          }
-        }
-      } //If !fieldsToGet.empty()
+      //TODO: Don't show it if m_view is false.
 
       //Select first record:
       Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
@@ -197,8 +133,9 @@ void Box_Data_List::on_adddel_user_requested_add()
     {
       index_field_to_edit = index_primary_key;
 
-      Field fieldPrimaryKey = m_Fields[index_primary_key].m_field;
-      if(fieldPrimaryKey.get_field_info().get_auto_increment())
+      Field fieldPrimaryKey;
+      bool test = get_field_primary_key(fieldPrimaryKey);
+      if(test && fieldPrimaryKey.get_auto_increment())
       {
         //Start editing in the first cell that is not the primary key:
         if(index_primary_key == 0)
@@ -210,17 +147,17 @@ void Box_Data_List::on_adddel_user_requested_add()
       }
     }
 
-    if(index_field_to_edit < m_Fields.size())
+    if(index_field_to_edit < m_FieldsShown.size())
     {
       guint treemodel_column = 0;
-      bool test = get_field_column_index(m_Fields[index_field_to_edit].get_name(), treemodel_column);
+      bool test = get_field_column_index(m_FieldsShown[index_field_to_edit].get_name(), treemodel_column);
       if(test)
         m_AddDel.select_item(iter, treemodel_column, true /* start_editing */);
     }
     else
     {
       //The only keys are non-editable, so just add a row:
-      on_adddel_user_added(iter);
+      on_adddel_user_added(iter, 0);
       m_AddDel.select_item(iter); //without start_editing.
       //g_warning("Box_Data_List::on_adddel_user_requested_add(): index_field_to_edit does not exist: %d", index_field_to_edit);
     }
@@ -249,26 +186,29 @@ void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterat
     dialog.set_transient_for(*get_app_window());
     dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
-    
+
     int response = dialog.run();
     if(response == Gtk::RESPONSE_OK)
     {
-      record_delete( get_primary_key_value(rowStart) );
+      const Gnome::Gda::Value primary_key_value = get_primary_key_value(rowStart);
+      record_delete(primary_key_value);
 
       //Remove the row:
       m_AddDel.remove_item(rowStart);
+
+      on_record_deleted(primary_key_value);
     }
   }
 }
 
-void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
+void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value)
 {
- g_warning("Box_Data_List::on_adddel_user_added");
   Gnome::Gda::Value primary_key_value;
 
   Field field_primary_key = m_AddDel.get_key_field();
 
-  if(field_primary_key.get_field_info().get_auto_increment())
+  //Get the new primary key value, if one is available now:
+  if(field_primary_key.get_auto_increment())
   {
     //Auto-increment is awkward (we can't get the last-generated ID) with postgres, so we auto-generate it ourselves;
     const Glib::ustring& strPrimaryKeyName = field_primary_key.get_name();
@@ -278,11 +218,11 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
   {
     //This only works when the primary key is already stored: primary_key_value = get_primary_key_value(row);
     LayoutItem_Field layout_item;
-    layout_item.set_name(field_primary_key.get_name());
     layout_item.m_field = field_primary_key;
     primary_key_value = get_entered_field_data(layout_item);
   }
 
+  //If no primary key value is available yet, then don't add the record yet:
   if(!GlomConversions::value_is_empty(primary_key_value))
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server(); //Keep it alive while we need the data_model.
@@ -295,17 +235,19 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
         m_AddDel.set_value_key(row, primary_key_value);
 
         //Show the primary key in the row, if the primary key is visible:
-        const Gnome::Gda::FieldAttributes fieldInfo = field_primary_key.get_field_info();
+
         //If it's an auto-increment, then get the value and show it:
-        if(fieldInfo.get_auto_increment())
+        if(field_primary_key.get_auto_increment())
         {
           LayoutItem_Field layout_item;
-          layout_item.set_name(field_primary_key.get_name());
           layout_item.m_field = field_primary_key;
           m_AddDel.set_value(row, layout_item, primary_key_value);
         }
 
         on_record_added(primary_key_value);
+
+        //Do any lookups, etc, trigerred by the change of value of the original changed field:
+        on_adddel_user_changed(row, col_with_first_value);
       }
       else
         handle_error();
@@ -349,6 +291,7 @@ void Box_Data_List::on_adddel_user_reordered_columns()
 void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col)
 {
   const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value(row);
+
   if(!GlomConversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
   {
     //Just update the record:
@@ -408,26 +351,33 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
       //Update the field in the record (the record with this primary key):
       const Gnome::Gda::Value field_value = m_AddDel.get_value(row, layout_field);
-      const Field& field = layout_field.m_field;
-      const Glib::ustring strFieldName = layout_field.get_name();
+      //const Field& field = layout_field.m_field;
+      //const Glib::ustring strFieldName = layout_field.get_name();
 
-      Glib::ustring strQuery = "UPDATE " + table_name;
-      strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
-      strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
-      bool bTest = Query_execute(strQuery);
+      bool bTest = set_field_value_in_database(row, layout_field, field_value, primary_key_field, primary_key_value);
+
+      //Glib::ustring strQuery = "UPDATE " + table_name;
+      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
+      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+      //bool bTest = Query_execute(strQuery);
       if(!bTest)
       {
         //Update failed.
         fill_from_database(); //Replace with correct values.
       }
+      /*
       else
       {
         //Get-and-set values for lookup fields, if this field triggers those relationships:
         do_lookups(row, layout_field, field_value, primary_key_field, primary_key_value);
 
+        //Recalculate calculated fields, if this field is used by them:
+        do_calculations(layout_field, primary_key_field, primary_key_value);
+
         //Update related fields, if this field is used in the relationship:
         refresh_related_fields(row, layout_field, field_value, primary_key_field, primary_key_value);
       }
+      */
     }
     catch(const std::exception& ex)
     {
@@ -438,14 +388,21 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
   {
     //This record probably doesn't exist yet.
     //Add new record, which will generate the primary key:
-
-    on_adddel_user_added(row);
+    //Actually, on_adddel_user_added() is usually just called directly in response to the user_added signal.
+    on_adddel_user_added(row, col);
 
     //TODO: When the primary is non auto-incrementing, this sets it again, though it was INSERTED in on_adddel_user_added().
     //That's harmless, but inefficient.
     const Gnome::Gda::Value primaryKeyValue = get_primary_key_value(row); //TODO_Value
     if(!(GlomConversions::value_is_empty(primaryKeyValue))) //If the Add succeeeded:
+    {
       on_adddel_user_changed(row, col); //Change this field in the new record.
+    }
+    else
+    {
+      //A field value was entered, but the record has not been added yet, because not enough information exists yet.
+       g_warning("Box_Data_List::on_adddel_user_changed(): debug: record not yet added.");
+    }
   }
 
 }
@@ -498,6 +455,7 @@ void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const Layout
    if(field_changed.get_has_relationship_name())
     return; //TODO: Handle these too.
 
+
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
    const Glib::ustring strFieldName = field_changed.get_name();
@@ -519,10 +477,11 @@ void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const Layout
        m_AddDel.set_value(row, layout_Item, value);
 
        //Add it to the database (even if it is not shown in the view)
-       Glib::ustring strQuery = "UPDATE " + m_strTableName;
-       strQuery += " SET " + field_lookup.get_name() + " = " + field_lookup.sql(value);
-       strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
-       Query_execute(strQuery);  //TODO: Handle errors
+       set_field_value_in_database(row, layout_Item, value, primary_key, primary_key_value); //Also does dependent lookups/recalcs.
+       //Glib::ustring strQuery = "UPDATE " + m_strTableName;
+       //strQuery += " SET " + field_lookup.get_name() + " = " + field_lookup.sql(value);
+       //strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
+       //Query_execute(strQuery);  //TODO: Handle errors
 
        //TODO: Handle lookups triggered by these fields (recursively)? TODO: Check for infinitely looping lookups.
      }
@@ -561,7 +520,7 @@ void Box_Data_List::on_details_nav_next()
     //Don't go past the last record:
     if( !m_AddDel.get_is_last_row(iter) )
     {
-      iter++;    
+      iter++;
       m_AddDel.select_item(iter);
 
       signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
@@ -637,8 +596,10 @@ guint Box_Data_List::get_records_count() const
   return m_AddDel.get_count();
 }
 
-void Box_Data_List::fill_column_titles()
+void Box_Data_List::create_layout()
 {
+  Box_Data::create_layout(); //Fills m_TableFields.
+
   const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
   if(pDoc)
   {
@@ -650,23 +611,45 @@ void Box_Data_List::fill_column_titles()
 
     Field field_primary_key;
     bool test = get_field_primary_key_for_table(m_strTableName, field_primary_key);
-    if(test)
-        m_AddDel.set_key_field(field_primary_key);
-
-
-    type_vecLayoutFields listFieldsToShow = get_fields_to_show();
-
-    //Add a column for each table field:
-    for(type_vecLayoutFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
+    if(!test)
     {
-      m_AddDel.add_column(*iter);
+      //g_warning("Box_Data_List::create_layout(): primary key not found.");
     }
+    else
+    {
+      m_AddDel.set_key_field(field_primary_key);
 
-    m_AddDel.set_columns_ready();
+      m_FieldsShown = get_fields_to_show();
+      //g_warning("m_Fields[0].m_field.get_name() = %s", m_FieldsShown[0].m_field.get_name().c_str());
+
+      //Add extra possibly-non-visible columns that we need:
+      //TODO: Only add it if it is not already there.
+      LayoutItem_Field layout_item;
+      layout_item.set_hidden();
+      Field field_key = m_AddDel.get_key_field();
+      layout_item.m_field = field_key;
+      m_FieldsShown.push_back(layout_item);
+
+
+      //Add a column for each table field:
+      for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
+      {
+        m_AddDel.add_column(*iter);
+      }
+
+      m_AddDel.set_where_clause(m_strWhereClause);
+
+      m_AddDel.set_columns_ready();
+     }
   }
 
 }
 
+void Box_Data_List::on_record_deleted(const Gnome::Gda::Value& /* primary_key_value */)
+{
+   //Overridden by Box_Data_List_Related.
+}
+
 void Box_Data_List::on_record_added(const Gnome::Gda::Value& /* strPrimaryKey */)
 {
   //Overridden by Box_Data_List_Related.
@@ -685,7 +668,7 @@ bool Box_Data_List::get_field_column_index(const Glib::ustring& field_name, guin
 
   //Get the index of the field with this name:
   guint i = 0;
-  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
+  for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
   {
     if(iter->get_name() == field_name)
     {
@@ -706,7 +689,7 @@ bool Box_Data_List::get_field_primary_key(Field& field) const
 
 bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
 {
-  return Box_Data::get_field_primary_key_index(m_Fields, field_column);
+  return Box_Data::get_field_primary_key_index(m_FieldsShown, field_column);
 }
 
 void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const LayoutGroup& /* group */)
