@@ -50,7 +50,11 @@ Box_Data_List_Related::Box_Data_List_Related()
 
   //Delete the dialog from the base class, because we don't use it.
   if(m_pDialogLayout)
+  {
+    remove_view(m_pDialogLayout);
     delete m_pDialogLayout;
+    m_pDialogLayout = 0;
+  }
 
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout_list_related");
   if(refXml)
@@ -61,6 +65,7 @@ Box_Data_List_Related::Box_Data_List_Related()
     {
       //Use the new dialog:
       m_pDialogLayoutRelated = dialog;
+      add_view(m_pDialogLayoutRelated); //Give it access to the document.
       m_pDialogLayoutRelated->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
     }
   }
@@ -71,6 +76,7 @@ Box_Data_List_Related::~Box_Data_List_Related()
 {
   if(m_pDialogLayoutRelated)
   {
+    remove_view(m_pDialogLayoutRelated);
     delete m_pDialogLayoutRelated;
     m_pDialogLayoutRelated = 0;
   }
@@ -79,6 +85,7 @@ Box_Data_List_Related::~Box_Data_List_Related()
 void Box_Data_List_Related::init_db_details(const Relationship& relationship)
 {
   m_relationship = relationship;
+  m_strTableName = relationship.get_to_table();
 
   m_Label.set_markup(Bakery::App_Gtk::util_bold_message( relationship.get_title_or_name() ));
 
@@ -87,10 +94,12 @@ void Box_Data_List_Related::init_db_details(const Relationship& relationship)
   {
     g_warning("Box_Data_List_Related::init_db_details(): key_field not found.");
   }
-  Box_Data_List::init_db_details(relationship.get_to_table());
+
+  Box_Data_List::init_db_details(relationship.get_to_table()); //Calls create_layout() and fill_from_database().
+
 }
 
-void Box_Data_List_Related::refresh_db_details(const Gnome::Gda::Value& foreign_key_value, const Gnome::Gda::Value& /* from_table_primary_key_value */)
+void Box_Data_List_Related::refresh_data_from_database(const Gnome::Gda::Value& foreign_key_value)
 {
   m_key_value = foreign_key_value;
 
@@ -98,16 +107,15 @@ void Box_Data_List_Related::refresh_db_details(const Gnome::Gda::Value& foreign_
   {
     Glib::ustring strWhereClause = m_key_field.get_name() + " = " + m_key_field.sql(m_key_value);
 
-    //g_warning("refresh_db_details(): where_clause=%s", strWhereClause.c_str());
-    Box_Data_List::refresh_db_details(strWhereClause);
+    //g_warning("refresh_data_from_database(): where_clause=%s", strWhereClause.c_str());
+    Box_Data_List::refresh_data_from_database(strWhereClause);
   }
   else
   {
-    g_warning("Box_Data_List_Related::refresh_db_details(): m_key_value is NULL.");
+    g_warning("Box_Data_List_Related::refresh_data_from_database(): m_key_value is NULL.");
   }
   //TODO: Clear the list if there is no key value?
 }
- 
 
 void Box_Data_List_Related::fill_from_database()
 {
@@ -122,7 +130,7 @@ void Box_Data_List_Related::fill_from_database()
     if(m_has_one_or_more_records) //This was set by Box_Data_List::fill_from_database().
     {
       //Is the to_field unique? If so, there can not be more than one.
-      if(m_key_field.get_field_info().get_unique_key()) //automatically true if it is a primary key
+      if(m_key_field.get_unique_key()) //automatically true if it is a primary key
         allow_add = false;
     }
 
@@ -136,7 +144,7 @@ void Box_Data_List_Related::fill_from_database()
 
     Box_DB_Table::fill_from_database();
 
-    fill_column_titles();
+    //create_layout();
 
     fill_end();
   }
@@ -235,8 +243,7 @@ void Box_Data_List_Related::on_adddel_user_added(const Gtk::TreeModel::iterator&
 
   bool bAllowAdd = true;
 
-  const Gnome::Gda::FieldAttributes fieldInfo = m_key_field.get_field_info();
-  if(fieldInfo.get_unique_key() || fieldInfo.get_primary_key())
+  if(m_key_field.get_unique_key() || m_key_field.get_primary_key())
   {
     if(m_AddDel.get_count() > 0) //If there is already 1 record
       bAllowAdd = false;
