@@ -19,8 +19,9 @@
  */
 
 #include "box_db_table_definition.h"
+#include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include "../../../config.h"
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Box_DB_Table_Definition::Box_DB_Table_Definition()
 {
@@ -35,19 +36,20 @@ Box_DB_Table_Definition::Box_DB_Table_Definition(BaseObjectType* cobject, const
 
 void Box_DB_Table_Definition::init()
 {
-  m_strHint = gettext("Click [Edit] to edit the field definition in more detail.\nUse the Mode menu to see Data or perform a Find.");
+  //m_strHint = _("Click [Edit] to edit the field definition in more detail.\nUse the Mode menu to see Data or perform a Find.");
 
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_field_definition_edit");
   if(refXml)
     refXml->get_widget_derived("window_field_definition_edit", m_pDialog);
+
   add_view(m_pDialog); //Give it access to the document.
 
   pack_start(m_AddDel);
-  m_colName = m_AddDel.add_column(gettext("Name"));
+  m_colName = m_AddDel.add_column(_("Name"));
 
-  m_colTitle = m_AddDel.add_column(gettext("Title"));
+  m_colTitle = m_AddDel.add_column(_("Title"));
 
-  m_colType = m_AddDel.add_column(gettext("Type"), AddDelColumnInfo::STYLE_Choices);
+  m_colType = m_AddDel.add_column(_("Type"), AddDelColumnInfo::STYLE_Choices);
   m_AddDel.set_column_width(m_colType, 100); //TODO: Auto-size columns.
 
   //Set Type choices:
@@ -77,39 +79,44 @@ void Box_DB_Table_Definition::init()
 
 Box_DB_Table_Definition::~Box_DB_Table_Definition()
 {
+  if(m_pDialog)
+  {
+    remove_view(m_pDialog);
+    delete m_pDialog;
+  }
 }
 
 void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& iter, const Field& field)
 {
   m_AddDel.set_value_key(iter, field.get_name());
-  
+ 
   m_AddDel.set_value(iter, m_colName, field.get_name());
 
   Glib::ustring title = field.get_title();
   m_AddDel.set_value(iter, m_colTitle, title);
 
-  Gnome::Gda::FieldAttributes fieldinfo = field.get_field_info();
-
   //Type:
-  Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(fieldinfo.get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
+  Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(field.get_field_info().get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
 
   Glib::ustring strType = Field::get_type_name_ui( fieldType );
   m_AddDel.set_value(iter, m_colType, strType);
 
   //Unique:
-  bool bUnique = fieldinfo.get_unique_key();
+  const bool bUnique = field.get_unique_key();
 
   m_AddDel.set_value(iter, m_colUnique, bUnique);
 
   //Primary Key:
-  bool bPrimaryKey = fieldinfo.get_primary_key();
+  const bool bPrimaryKey = field.get_primary_key();
   m_AddDel.set_value(iter, m_colPrimaryKey, bPrimaryKey);
 }
 
 void Box_DB_Table_Definition::fill_from_database()
 {
   Box_DB_Table::fill_from_database();
-  
+
+  fill_fields();
+
   try
   {
     //Fields:
@@ -143,7 +150,7 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
     if(bTest)
     {
       //Show the new field (fill in the other cells):
-      
+
       fill_fields();
 
       //fill_from_database(); //We can not change the structure in a cell renderer signal handler.
@@ -157,9 +164,9 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
       Gnome::Gda::FieldAttributes field_info = field.get_field_info();
       field_info.set_gdatype( Field::get_gda_type_for_glom_type(Field::TYPE_NUMERIC) );
       field.set_field_info(field_info);
-      
+
       fill_field_row(row, field);
-      
+
       //m_AddDel.select_item(row, m_colTitle, true); //Start editing the title
     }
   }
@@ -211,8 +218,8 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
        //Refuse to edit field definitions that were not created by glom:
        if(Field::get_glom_type_for_gda_type( m_Field_BeingEdited.get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
        {
-         Gtk::MessageDialog dialog(gettext("<b>Invalid database structure</b>"), true);
-         dialog.set_secondary_text(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+         Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
+         dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
          //TODO: dialog.set_transient_for(*get_application());
          dialog.run();
        }
@@ -312,8 +319,8 @@ void Box_DB_Table_Definition::on_Properties_apply()
     //Refuse to edit field definitions that were not created by glom:
     if( Field::get_glom_type_for_gda_type(m_Field_BeingEdited.get_field_info().get_gdatype()) == Field::TYPE_INVALID )
     {
-      Gtk::MessageDialog dialog(gettext("<b>Invalid database structure</b>"), true);
-         dialog.set_secondary_text(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
+         dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
       //TODO: dialog.set_transient_for(*get_app_window());
       dialog.run();
     }
@@ -338,32 +345,11 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
 
   postgres_change_column(fieldOld, field);
 
-   /*
-    //Get a data model:
-    Gnome::Gda::Command command("SELECT * FROM " + m_strTableName + " LIMIT 1"); //LIMIT 1 means we get only the first row.
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model = gda_connection->execute_single_command(command);
-
-    if(data_model)
-    {
-      Gnome::Gda::FieldAttributes field_info = field.get_field_info();
-      bool test = data_model->update_column(field_info);
-      if(!test)
-      {
-        g_warning("Box_DB_Table_Definition::change_definition(): libgda update_column() failed.");
-        handle_error();
-      }
-      else
-        g_warning("Box_DB_Table_Definition::change_definition(): libgda says that update_column() succeeded.");
-    }
-  }
-  */
- 
 
    //MySQL does this all with ALTER_TABLE, with "CHANGE" followed by the same details used with "CREATE TABLE",
    //MySQL also makes it easier to change the type.
    // but Postgres uses various subcommands, such as  "ALTER COLUMN", and "RENAME".
 
- 
   //Extra Glom field definitions:
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
   if(pDoc)
@@ -540,12 +526,12 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
 
 void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_old, const Field& field, bool set_anyway)
 {
-  Gnome::Gda::FieldAttributes field_info = field.get_field_info();
-  Gnome::Gda::FieldAttributes field_info_old = field_old.get_field_info();
+  //Gnome::Gda::FieldAttributes field_info = field.get_field_info();
+  //Gnome::Gda::FieldAttributes field_info_old = field_old.get_field_info();
 
-  if(field_info.get_name() != field_info_old.get_name())
+  if(field.get_name() != field_old.get_name())
   {
-     Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " RENAME COLUMN " + field_info_old.get_name() + " TO " + field_info.get_name() );
+     Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " RENAME COLUMN " + field_old.get_name() + " TO " + field.get_name() );
      if(!datamodel)
      {
        handle_error();
@@ -553,14 +539,14 @@ void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_o
      }
   }
 
-  if(set_anyway || (field_info.get_primary_key() != field_info_old.get_primary_key()))
+  if(set_anyway || (field.get_primary_key() != field_old.get_primary_key()))
   {
     //TODO: Check that there is only one primary key.
     Glib::ustring add_or_drop = "ADD";
-    if(field_info_old.get_primary_key() == false)
+    if(field_old.get_primary_key() == false)
       add_or_drop = "DROP";
 
-    Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " " + add_or_drop + " PRIMARY KEY (" + field_info.get_name() + ")");
+    Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " " + add_or_drop + " PRIMARY KEY (" + field.get_name() + ")");
     if(!datamodel)
     {
       handle_error();
@@ -568,9 +554,9 @@ void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_o
     }
   }
 
-  if( !field_info.get_primary_key() ) //Postgres automatically makes primary keys unique, so we do not need to do that separately if we have already made it a primary key
+  if( !field.get_primary_key() ) //Postgres automatically makes primary keys unique, so we do not need to do that separately if we have already made it a primary key
   {
-    if(set_anyway || (field_info.get_unique_key() != field_info_old.get_unique_key()))
+    if(set_anyway || (field.get_unique_key() != field_old.get_unique_key()))
     {
        /* TODO: Is there an easier way than adding an index manually?
        Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " RENAME COLUMN " + field_info_old.get_name() + " TO " + field_info.get_name() );
@@ -582,14 +568,14 @@ void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_o
        */
     }
 
-    Gnome::Gda::Value default_value = field_info.get_default_value();
-    Gnome::Gda::Value default_value_old = field_info_old.get_default_value();
+    Gnome::Gda::Value default_value = field.get_default_value();
+    Gnome::Gda::Value default_value_old = field_old.get_default_value();
 
-    if(!field_info.get_auto_increment()) //Postgres auto-increment fields have special code as their default values.
+    if(!field.get_auto_increment()) //Postgres auto-increment fields have special code as their default values.
     {
       if(set_anyway || (default_value != default_value_old))
       {
-        Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " ALTER COLUMN "+ field_info.get_name() + " SET DEFAULT " + field.sql(field_info.get_default_value()) );
+        Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "ALTER TABLE " + m_strTableName + " ALTER COLUMN "+ field.get_name() + " SET DEFAULT " + field.sql(field.get_default_value()) );
         if(!datamodel)
         {
           handle_error();
